﻿Imports System.Collections.Generic
Imports System.Data
Imports System.Linq
Imports AdventureWorksModel

Public Class DataAccess

    Public Shared Function GetPagedCustomers(ByVal skip As Integer, ByVal take As Integer) As PagedList(Of Customer)
        Using context = New AdventureWorksLTEntities
            'refactor consideration...make this a pre-compiled query
            Dim query = context.Customers.Include("SalesOrderHeaders").Where(
                Function(c) c.SalesOrderHeaders.Any()).OrderBy(
                Function(c) c.CompanyName + c.LastName + c.FirstName)

            'getting this count every time in case others are adding or deleting customers
            'decide for yourself in your app
            Dim customerCount = query.Count()

            Dim customers = query.Skip(skip).Take(take).ToList()

            Return New PagedList(Of Customer) With
                   {.Entities = customers,
                    .HasNext = (skip + 10 < customerCount),
                    .HasPrevious = (skip > 0)}
        End Using
    End Function

    Public Shared Function GetCustomerById(ByVal id As Integer) As Customer
        Using context = New AdventureWorksLTEntities
            'refactor consideration...make this a pre-compiled query
            Dim query =
             From c In context.Customers
             Where c.CustomerID = id
             Select c
            Return query.SingleOrDefault() 'non-demo app should consider exception handling
        End Using
    End Function

    Public Shared Sub UpdateCustomer(ByVal customer As Customer)
        'this update is simplified because I placed some non-nullable properties
        'that I don't want to work with into a separate entity using the "table splitting"
        'mapping feature of EDM. If you were doing an insert, you'll have to deal with those
        'properties from the related entity. Way beyond the scope of the article that this
        'demo is for, however.
        Using context = New AdventureWorksLTEntities
            context.Customers.Attach(customer)
            context.ObjectStateManager.ChangeObjectState(customer, EntityState.Modified)
            context.SaveChanges() 'non-demo app should consider exception handling
        End Using
    End Sub
End Class

Public Class PagedList(Of T)
    Public Property HasNext As Boolean
    Public Property HasPrevious As Boolean
    Public Property Entities As List(Of T)
End Class